<?php

namespace Drupal\ecomail;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Component\Serialization\Json;
use Drupal\Core\Logger\LoggerChannelFactoryInterface;
use Drupal\key\KeyRepositoryInterface;

/**
 * Class EcomailClientWrapper.
 */
class EcomailClientWrapper implements EcomailClientWrapperInterface {

  /**
   * Drupal\Core\Config\ConfigFactoryInterface.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $config;

  /**
   * Ecomail php wrapper.
   *
   * @var \Ecomail
   *
   * @see https://github.com/Ecomailcz/ecomail-php
   */
  protected $ecomail;

  /**
   * Drupal\Core\Logger\LoggerChannelFactoryInterface.
   *
   * @var \Drupal\Core\Logger\LoggerChannelFactoryInterface
   */
  protected $logger;

  /**
   * @var KeyRepositoryInterface $keyRepository
   */
  protected $keyRepository;

  /**
   * Ecomail constructor.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config
   *   Config.
   * @param \Drupal\key\KeyRepositoryInterface
   * @param \Drupal\Core\Logger\LoggerChannelFactoryInterface $logger_factory
   *   Logger.
   */
  public function __construct(ConfigFactoryInterface $config, KeyRepositoryInterface $key_repository, LoggerChannelFactoryInterface $logger_factory) {
    $this->config = $config->get('ecomail.settings');
    $this->keyRepository = $key_repository;
    $this->logger = $logger_factory;
    $this->ecomail = new \Ecomail($this->getAPIKey());
  }

  /**
   * {@inheritdoc}
   */
  public function getListsCollection() {
    return $this->processResponse($this->ecomail->getListsCollection());
  }

  /**
   * {@inheritdoc}
   */
  public function addListCollection(array $data)
  {
    return $this->processResponse($this->ecomail->addListCollection($data));
  }

  /**
   * {@inheritdoc}
   */
  public function showList($list_id) {
    return $this->processResponse($this->ecomail->showList($list_id));
  }

  /**
   * {@inheritdoc}
   */
  public function updateList($list_id, array $data) {
    return $this->processResponse($this->ecomail->updateList($list_id, $data));
  }

  /**
   * {@inheritdoc}
   */
  public function getSubscribers($list_id) {
    return $this->processResponse($this->ecomail->getSubscribers($list_id));
  }

  /**
   * {@inheritdoc}
   */
  public function getSubscriber($list_id, $email) {
    return $this->processResponse($this->ecomail->getSubscriber($list_id, $email));
  }

  /**
   * {@inheritdoc}
   */
  public function getSubscriberList($email) {
    return $this->processResponse($this->ecomail->getSubscriberList($email));
  }

  /**
   * {@inheritdoc}
   */
  public function addSubscriber($list_id, array $data) {
    return $this->processResponse($this->ecomail->addSubscriber($list_id, $data));
  }

  /**
   * {@inheritdoc}
   */
  public function removeSubscriber($list_id, array $data) {
    return $this->processResponse($this->ecomail->removeSubscriber($list_id, $data));
  }

  /**
   * {@inheritdoc}
   */
  public function updateSubscriber($list_id, array $data) {
    return $this->processResponse($this->ecomail->updateSubscriber($list_id, $data));
  }

  /**
   * {@inheritdoc}
   */
  public function addSubscriberBulk($list_id, array $data)
  {
    return $this->processResponse($this->ecomail->addSubscriberBulk($list_id, $data));
  }

  /**
   * {@inheritdoc}
   */
  public function deleteSubscriber($email)
  {
    return $this->processResponse($this->ecomail->deleteSubscriber($email));
  }

  /**
   * {@inheritdoc}
   */
  public function listCampaigns($filters = null)
  {
    return $this->processResponse($this->ecomail->listCampaigns());
  }

  /**
   * {@inheritdoc}
   */
  public function addCampaign(array $data)
  {
    return $this->processResponse($this->ecomail->addCampaign($data));
  }

  /**
   * {@inheritdoc}
   */
  public function updateCampaign($campaign_id, array $data)
  {
    return $this->processResponse($this->ecomail->updateCampaign($campaign_id, $data));
  }

  /**
   * {@inheritdoc}
   */
  public function sendCampaign($campaign_id)
  {
    return $this->processResponse($this->ecomail->sendCampaign($campaign_id));
  }

  /**
   * {@inheritdoc}
   */
  public function getCampaignStats($campaign_id)
  {
    return $this->processResponse($this->ecomail->getCampaignStats($campaign_id));
  }

  /**
   * {@inheritdoc}
   */
  public function getCampaignStatsDetail($campaignId, $queryParams = array())
  {
    return $this->processResponse($this->ecomail->getCampaignStatsDetail($campaignId));
  }

  /**
   * {@inheritdoc}
   */
  public function listAutomations()
  {
    return $this->processResponse($this->ecomail->listAutomations());
  }

  /**
   * {@inheritdoc}
   */
  public function triggerAutomation($automation_id, array $data)
  {
    return $this->processResponse($this->ecomail->triggerAutomation($automation_id, $data));
  }

  /**
   * {@inheritdoc}
   */
  public function getPipelineStats($pipelineId)
  {
    return $this->processResponse($this->ecomail->getPipelineStats($pipelineId));
  }

  /**
   * {@inheritdoc}
   */
  public function getPipelineStatsDetail($pipelineId, $queryParams = array())
  {
    return $this->processResponse($this->ecomail->getPipelineStatsDetail($pipelineId));
  }

  /**
   * {@inheritdoc}
   */
  public function createTemplate(array $data)
  {
    return $this->processResponse($this->ecomail->createTemplate($data));
  }

  /**
   * {@inheritdoc}
   */
  public function listDomains()
  {
    return $this->processResponse($this->ecomail->listDomains());
  }

  /**
   * {@inheritdoc}
   */
  public function createDomain(array $data)
  {
    return $this->processResponse($this->ecomail->createDomain($data));
  }

  /**
   * {@inheritdoc}
   */
  public function deleteDomain($id)
  {
    return $this->processResponse($this->ecomail->deleteDomain($id));
  }

  /**
   * {@inheritdoc}
   */
  public function sendTransactionalEmail(array $data)
  {
    return $this->processResponse($this->ecomail->sendTransactionalEmail($data));
  }

  /**
   * {@inheritdoc}
   */
  public function sendTransactionalTemplate(array $data)
  {
    return $this->processResponse($this->ecomail->sendTransactionalTemplate($data));
  }

  /**
   * {@inheritdoc}
   */
  public function getTransactionalStats()
  {
    return $this->processResponse($this->ecomail->getTransactionalStats());
  }

  /**
   * {@inheritdoc}
   */
  public function getTransactionalStatsDOI()
  {
    return $this->processResponse($this->ecomail->getTransactionalStatsDOI());
  }

  /**
   * {@inheritdoc}
   */
  public function createNewTransaction(array $data)
  {
    return $this->processResponse($this->ecomail->createNewTransaction($data));
  }

  /**
   * {@inheritdoc}
   */
  public function createBulkTransactions(array $data)
  {
    return $this->processResponse($this->ecomail->createBulkTransactions($data));
  }

  /**
   * {@inheritdoc}
   */
  public function updateTransaction($transaction_id, array $data)
  {
    return $this->processResponse($this->ecomail->updateTransaction($transaction_id, $data));
  }

  /**
   * {@inheritdoc}
   */
  public function deleteTransaction($transaction_id)
  {
    return $this->processResponse($this->ecomail->deleteTransaction($transaction_id));
  }

  /**
   * {@inheritdoc}
   */
  public function refreshProductFeed($feedId)
  {
    return $this->processResponse($this->ecomail->refreshProductFeed($feedId));
  }

  /**
   * {@inheritdoc}
   */
  public function refreshDataFeed($feedId)
  {
    return $this->processResponse($this->ecomail->refreshDataFeed($feedId));
  }

  /**
   * {@inheritdoc}
   */
  public function addEvent(array $data)
  {
    return $this->processResponse($this->ecomail->addEvent($data));
  }

  /**
   * {@inheritdoc}
   */
  public function search($query)
  {
    return $this->processResponse($this->ecomail->search($query));
  }

  /**
   * {@inheritdoc}
   */
  public function importCoupons(array $data)
  {
    return $this->processResponse($this->ecomail->importCoupons($data));
  }

  /**
   * {@inheritdoc}
   */
  public function deleteCoupons(array $data)
  {
    return $this->processResponse($this->ecomail->deleteCoupons($data));
  }

  /**
   * Helper method to process response from API.
   *
   * @param mixed $response
   *   Response from API call.
   *
   * @return mixed|null
   *   Return decoded data or NULL.
   */
  protected function processResponse($response) {
    if (is_array($response) || is_object($response)) {
      return $response;
    }
    if (is_string($response)) {
      $decoded_response = Json::decode($response);

      if ($decoded_response) {
        return $decoded_response;
      }
    }

    $this->logger->get('ecomail_webform')->error('Response from API: ' . Json::encode($response));
    return NULL;
  }

  protected function getAPIKey() {
    $key_name = $this->config->get('api_key');
    return $this->keyRepository->getKey($key_name)->getKeyValue();
  }

}
